@@ -1,5 +1,5 @@
 /*
- * Copyright 2019 OARC, Inc.
+ * Copyright 2019-2020 OARC, Inc.
  * Copyright 2017-2018 Akamai Technologies
  * Copyright 2006-2016 Nominum, Inc.
  * All rights reserved.
@@ -58,9 +58,7 @@
 #include "log.h"
 #include "opt.h"
 
-#ifndef ISC_SHA256_DIGESTLENGTH
-#define ISC_SHA256_DIGESTLENGTH 32U
-#endif
+#define _STATIC_DIGEST_BUFSIZE 128
 
 #define WHITESPACE " \t\n"
 
@@ -138,11 +136,15 @@ perf_dns_createctx(bool updates)
     if (!updates)
         return NULL;
 
-    mctx   = NULL;
+    mctx = NULL;
+#ifdef HAVE_ISC_MEM_CREATE_RESULT
     result = isc_mem_create(0, 0, &mctx);
     if (result != ISC_R_SUCCESS)
         perf_log_fatal("creating memory context: %s",
             isc_result_totext(result));
+#else
+    isc_mem_create(&mctx);
+#endif
 
     ctx = isc_mem_get(mctx, sizeof(*ctx));
     if (ctx == NULL) {
@@ -295,7 +297,11 @@ perf_dns_parsetsigkey(const char* arg, isc_mem_t* mctx)
         exit(1);
     }
 
-/* Name */
+    if (tsigkey->digestlen > _STATIC_DIGEST_BUFSIZE) {
+        perf_log_fatal("unable to setup TSIG algorithm %.*s, digest buffer too small, please report to %s", alglen, alg, PACKAGE_BUGREPORT);
+    }
+
+    /* Name */
 
 #ifdef dns_fixedname_init
     dns_fixedname_init(&tsigkey->fname);
@@ -371,9 +377,13 @@ perf_dns_parseednsoption(const char* arg, isc_mem_t* mctx)
 
     option->mctx   = mctx;
     option->buffer = NULL;
-    result         = isc_buffer_allocate(mctx, &option->buffer, strlen(value) / 2 + 4);
+#ifdef HAVE_ISC_BUFFER_ALLOCATE_RESULT
+    result = isc_buffer_allocate(mctx, &option->buffer, strlen(value) / 2 + 4);
     if (result != ISC_R_SUCCESS)
         perf_log_fatal("out of memory");
+#else
+    isc_buffer_allocate(mctx, &option->buffer, strlen(value) / 2 + 4);
+#endif
 
     result = isc_parse_uint16(&code, copy, 0);
     if (result != ISC_R_SUCCESS) {
@@ -623,7 +633,7 @@ add_tsig(isc_buffer_t* packet, perf_dnstsigkey_t* tsigkey)
     unsigned char  tmpdata[512];
     isc_buffer_t   tmp;
     uint32_t       now;
-    unsigned char  digest[ISC_SHA256_DIGESTLENGTH];
+    unsigned char  digest[_STATIC_DIGEST_BUFSIZE];
 
     hmac_init(tsigkey, &hmac);
     now = time(NULL);
@@ -907,7 +917,7 @@ build_update(perf_dnsctx_t* ctx, const isc_textregion_t* record,
         if (token_equals(&token, "send")) {
             break;
         } else if (token_equals(&token, "add")) {
-            result = read_update_line(ctx, &input, str, zname,
+            result    = read_update_line(ctx, &input, str, zname,
                 true, true, true,
                 true, oname, &ttl, &rdtype,
                 &rdata, &rdatabuf);
@@ -922,7 +932,7 @@ build_update(perf_dnsctx_t* ctx, const isc_textregion_t* record,
                 rdclass = dns_rdataclass_none;
             else
                 rdclass = dns_rdataclass_any;
-            is_update   = true;
+            is_update = true;
         } else if (token_equals(&token, "require")) {
             result = read_update_line(ctx, &input, str, zname,
                 false, false, true,
@@ -932,9 +942,9 @@ build_update(perf_dnsctx_t* ctx, const isc_textregion_t* record,
                 rdclass = dns_rdataclass_in;
             else
                 rdclass = dns_rdataclass_any;
-            is_update   = false;
+            is_update = false;
         } else if (token_equals(&token, "prohibit")) {
-            result = read_update_line(ctx, &input, str, zname,
+            result    = read_update_line(ctx, &input, str, zname,
                 false, false, false,
                 false, oname, &ttl,
                 &rdtype, &rdata, &rdatabuf);
