@@ -1,5 +1,5 @@
 /*
- * Copyright 2019 OARC, Inc.
+ * Copyright 2019-2020 OARC, Inc.
  * Copyright 2017-2018 Akamai Technologies
  * Copyright 2006-2016 Nominum, Inc.
  * All rights reserved.
@@ -39,6 +39,7 @@
 #include "net.h"
 #include "opt.h"
 #include "os.h"
+#include "strerror.h"
 
 #define TCP_RECV_BUF_SIZE (16 * 1024)
 #define TCP_SEND_BUF_SIZE (4 * 1024)
@@ -119,7 +120,7 @@ struct perf_net_socket perf_net_opensocket(enum perf_net_mode mode, const isc_so
     int                    port;
     int                    ret;
     int                    flags;
-    struct perf_net_socket sock = {.mode = mode, .is_ready = 1 };
+    struct perf_net_socket sock = { .mode = mode, .is_ready = 1 };
 
     family = isc_sockaddr_pf(server);
 
@@ -136,7 +137,8 @@ struct perf_net_socket perf_net_opensocket(enum perf_net_mode mode, const isc_so
             perf_log_fatal("pthread_mutex_init() failed");
         }
         if ((sock.fd = socket(family, SOCK_STREAM, 0)) < 0) {
-            perf_log_fatal("socket: %s", strerror(errno));
+            char __s[256];
+            perf_log_fatal("socket: %s", perf_strerror_r(errno, __s, sizeof(__s)));
         }
         if (!ssl_ctx) {
 #ifdef HAVE_TLS_CLIENT_METHOD
@@ -161,8 +163,10 @@ struct perf_net_socket perf_net_opensocket(enum perf_net_mode mode, const isc_so
         perf_log_fatal("perf_net_opensocket(): invalid mode");
     }
 
-    if (sock.fd == -1)
-        perf_log_fatal("socket: %s", strerror(errno));
+    if (sock.fd == -1) {
+        char __s[256];
+        perf_log_fatal("socket: %s", perf_strerror_r(errno, __s, sizeof(__s)));
+    }
 
 #if defined(AF_INET6) && defined(IPV6_V6ONLY)
     if (family == AF_INET6) {
@@ -183,8 +187,10 @@ struct perf_net_socket perf_net_opensocket(enum perf_net_mode mode, const isc_so
         isc_sockaddr_setport(&tmp, port);
     }
 
-    if (bind(sock.fd, &tmp.type.sa, tmp.length) == -1)
-        perf_log_fatal("bind: %s", strerror(errno));
+    if (bind(sock.fd, &tmp.type.sa, tmp.length) == -1) {
+        char __s[256];
+        perf_log_fatal("bind: %s", perf_strerror_r(errno, __s, sizeof(__s)));
+    }
 
     if (bufsize > 0) {
         bufsize *= 1024;
@@ -212,7 +218,8 @@ struct perf_net_socket perf_net_opensocket(enum perf_net_mode mode, const isc_so
             if (errno == EINPROGRESS) {
                 sock.is_ready = 0;
             } else {
-                perf_log_fatal("connect() failed: %s", strerror(errno));
+                char __s[256];
+                perf_log_fatal("connect() failed: %s", perf_strerror_r(errno, __s, sizeof(__s)));
             }
         }
         sock.recvbuf   = malloc(TCP_RECV_BUF_SIZE);
@@ -235,8 +242,6 @@ ssize_t perf_net_recv(struct perf_net_socket* sock, void* buf, size_t len, int f
         uint16_t dnslen, dnslen2;
 
         if (!sock->have_more) {
-            struct pollfd p = { sock->fd, POLLIN, 0 };
-
             if (pthread_mutex_lock(&sock->lock)) {
                 perf_log_fatal("pthread_mutex_lock() failed");
             }
@@ -247,16 +252,6 @@ ssize_t perf_net_recv(struct perf_net_socket* sock, void* buf, size_t len, int f
                 errno = EAGAIN;
                 return -1;
             }
-#ifdef HAVE_SSL_HAS_PENDING
-            if (!SSL_has_pending(sock->ssl)) {
-#else
-            if (!SSL_pending(sock->ssl)) {
-#endif
-                if (poll(&p, 1, 10) == 1 && !(p.revents & POLLIN)) {
-                    errno = EAGAIN;
-                    return -1;
-                }
-            }
 
             n = SSL_read(sock->ssl, sock->recvbuf + sock->at, TCP_RECV_BUF_SIZE - sock->at);
             if (n < 0) {
@@ -295,7 +290,7 @@ ssize_t perf_net_recv(struct perf_net_socket* sock, void* buf, size_t len, int f
         if (sock->at > 2) {
             memcpy(&dnslen2, sock->recvbuf, 2);
             dnslen2 = ntohs(dnslen2);
-            if (sock->at >= dnslen + 2) {
+            if (sock->at >= dnslen2 + 2) {
                 sock->have_more = 1;
                 return dnslen;
             }
@@ -309,15 +304,12 @@ ssize_t perf_net_recv(struct perf_net_socket* sock, void* buf, size_t len, int f
         uint16_t dnslen, dnslen2;
 
         if (!sock->have_more) {
-            // Poll TCP sock to not choke it with recv(), increased throughput by ~50%
-            struct pollfd p = { sock->fd, POLLIN, 0 };
-            if (poll(&p, 1, 10) == 1 && !(p.revents & POLLIN)) {
-                errno = EAGAIN;
-                return -1;
-            }
-
             n = recv(sock->fd, sock->recvbuf + sock->at, TCP_RECV_BUF_SIZE - sock->at, flags);
             if (n < 0) {
+                if (errno == ECONNRESET) {
+                    // Treat connection reset like try again until reconnection features are in
+                    errno = EAGAIN;
+                }
                 return n;
             }
             sock->at += n;
@@ -340,7 +332,7 @@ ssize_t perf_net_recv(struct perf_net_socket* sock, void* buf, size_t len, int f
         if (sock->at > 2) {
             memcpy(&dnslen2, sock->recvbuf, 2);
             dnslen2 = ntohs(dnslen2);
-            if (sock->at >= dnslen + 2) {
+            if (sock->at >= dnslen2 + 2) {
                 sock->have_more = 1;
                 return dnslen;
             }
