@@ -1,5 +1,5 @@
 /*
- * Copyright 2019 OARC, Inc.
+ * Copyright 2019-2020 OARC, Inc.
  * Copyright 2017-2018 Akamai Technologies
  * Copyright 2006-2016 Nominum, Inc.
  * All rights reserved.
@@ -17,6 +17,8 @@
  * limitations under the License.
  */
 
+#include "net.h"
+
 #ifndef PERF_OS_H
 #define PERF_OS_H 1
 
@@ -28,10 +30,14 @@ void perf_os_blocksignal(int sig, bool block);
 void perf_os_handlesignal(int sig, void (*handler)(int));
 
 isc_result_t
-perf_os_waituntilreadable(int fd, int pipe_fd, int64_t timeout);
+perf_os_waituntilreadable(struct perf_net_socket* sock, int pipe_fd, int64_t timeout);
 
 isc_result_t
-perf_os_waituntilanyreadable(int* fds, unsigned int nfds, int pipe_fd,
+perf_os_waituntilanyreadable(struct perf_net_socket* socks, unsigned int nfds, int pipe_fd,
+    int64_t timeout);
+
+isc_result_t
+perf_os_waituntilanywritable(struct perf_net_socket* socks, unsigned int nfds, int pipe_fd,
     int64_t timeout);
 
 #endif
