@@ -1,5 +1,5 @@
 /*
- * Copyright 2019 OARC, Inc.
+ * Copyright 2019-2020 OARC, Inc.
  * Copyright 2017-2018 Akamai Technologies
  * Copyright 2006-2016 Nominum, Inc.
  * All rights reserved.
@@ -29,8 +29,11 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-
 #include <sys/time.h>
+#include <openssl/ssl.h>
+#include <openssl/conf.h>
+#include <openssl/err.h>
+#include <signal.h>
 
 #include <isc/buffer.h>
 #include <isc/file.h>
@@ -50,6 +53,7 @@
 #include "net.h"
 #include "opt.h"
 #include "util.h"
+#include "os.h"
 
 /*
  * Global stuff
@@ -57,6 +61,8 @@
 
 #define DEFAULT_SERVER_NAME "127.0.0.1"
 #define DEFAULT_SERVER_PORT 53
+#define DEFAULT_SERVER_TLS_PORT 853
+#define DEFAULT_SERVER_PORTS "udp/tcp 53 or tls 853"
 #define DEFAULT_LOCAL_PORT 0
 #define DEFAULT_SOCKET_BUFFER 32
 #define DEFAULT_TIMEOUT 45
@@ -64,6 +70,8 @@
 
 #define MAX_INPUT_DATA (4 * 1024)
 
+#define TIMEOUT_CHECK_TIME 5000000
+
 struct query_info;
 
 typedef ISC_LIST(struct query_info) query_list;
@@ -89,10 +97,13 @@ static query_info* queries;
 
 static isc_mem_t* mctx;
 
-static isc_sockaddr_t server_addr;
-static isc_sockaddr_t local_addr;
-static unsigned int   nsocks;
-static int*           socks;
+static isc_sockaddr_t          server_addr;
+static isc_sockaddr_t          local_addr;
+static unsigned int            nsocks;
+static struct perf_net_socket* socks;
+static enum perf_net_mode      mode;
+
+static int dummypipe[2];
 
 static uint64_t query_timeout;
 static bool     edns;
@@ -190,6 +201,8 @@ static uint64_t sustain_phase_began, wait_phase_began;
 
 static perf_dnstsigkey_t* tsigkey;
 
+static bool verbose;
+
 static char*
 stringify(double value, int precision)
 {
@@ -204,7 +217,7 @@ setup(int argc, char** argv)
 {
     const char*  family      = NULL;
     const char*  server_name = DEFAULT_SERVER_NAME;
-    in_port_t    server_port = DEFAULT_SERVER_PORT;
+    in_port_t    server_port = 0;
     const char*  local_name  = NULL;
     in_port_t    local_port  = DEFAULT_LOCAL_PORT;
     const char*  filename    = NULL;
@@ -212,17 +225,21 @@ setup(int argc, char** argv)
     int          sock_family;
     unsigned int bufsize;
     unsigned int i;
-    isc_result_t result;
+    const char*  _mode = 0;
 
-    result = isc_mem_create(0, 0, &mctx);
+#ifdef HAVE_ISC_MEM_CREATE_RESULT
+    isc_result_t result = isc_mem_create(0, 0, &mctx);
     if (result != ISC_R_SUCCESS)
         perf_log_fatal("creating memory context: %s",
             isc_result_totext(result));
+#else
+    isc_mem_create(&mctx);
+#endif
 
     dns_result_register();
 
     sock_family     = AF_UNSPEC;
-    server_port     = DEFAULT_SERVER_PORT;
+    server_port     = 0;
     local_port      = DEFAULT_LOCAL_PORT;
     bufsize         = DEFAULT_SOCKET_BUFFER;
     query_timeout   = DEFAULT_TIMEOUT * MILLION;
@@ -231,15 +248,18 @@ setup(int argc, char** argv)
     bucket_interval = DEFAULT_BUCKET_INTERVAL * MILLION;
     max_outstanding = DEFAULT_MAX_OUTSTANDING;
     nsocks          = 1;
+    mode            = sock_udp;
+    verbose         = false;
 
     perf_opt_add('f', perf_opt_string, "family",
         "address family of DNS transport, inet or inet6", "any",
         &family);
+    perf_opt_add('M', perf_opt_string, "mode", "set transport mode: udp, tcp or tls", "udp", &_mode);
     perf_opt_add('s', perf_opt_string, "server_addr",
         "the server to query", DEFAULT_SERVER_NAME, &server_name);
     perf_opt_add('p', perf_opt_port, "port",
         "the port on which to query the server",
-        stringify(DEFAULT_SERVER_PORT, 0), &server_port);
+        DEFAULT_SERVER_PORTS, &server_port);
     perf_opt_add('a', perf_opt_string, "local_addr",
         "the local address from which to send queries", NULL,
         &local_name);
@@ -282,9 +302,19 @@ setup(int argc, char** argv)
     perf_opt_add('q', perf_opt_uint, "num_outstanding",
         "the maximum number of queries outstanding",
         stringify(DEFAULT_MAX_OUTSTANDING, 0), &max_outstanding);
+    perf_opt_add('v', perf_opt_boolean, NULL,
+        "verbose: report additional information to stdout",
+        NULL, &verbose);
 
     perf_opt_parse(argc, argv);
 
+    if (_mode != 0)
+        mode = perf_net_parsemode(_mode);
+
+    if (!server_port) {
+        server_port = mode == sock_tls ? DEFAULT_SERVER_TLS_PORT : DEFAULT_SERVER_PORT;
+    }
+
     if (max_outstanding > nsocks * DEFAULT_MAX_OUTSTANDING)
         perf_log_fatal("number of outstanding packets (%u) must not "
                        "be more than 64K per client",
@@ -319,11 +349,11 @@ setup(int argc, char** argv)
     if (tsigkey_str != NULL)
         tsigkey = perf_dns_parsetsigkey(tsigkey_str, mctx);
 
-    socks = isc_mem_get(mctx, nsocks * sizeof(int));
+    socks = isc_mem_get(mctx, nsocks * sizeof(*socks));
     if (socks == NULL)
         perf_log_fatal("out of memory");
-    for (i       = 0; i < nsocks; i++)
-        socks[i] = perf_net_opensocket(&server_addr, &local_addr, i, bufsize);
+    for (i = 0; i < nsocks; i++)
+        socks[i] = perf_net_opensocket(mode, &server_addr, &local_addr, i, bufsize);
 }
 
 static void
@@ -333,10 +363,13 @@ cleanup(void)
 
     perf_datafile_close(&input);
     for (i = 0; i < nsocks; i++)
-        (void)close(socks[i]);
-    isc_mem_put(mctx, socks, nsocks * sizeof(int));
+        (void)perf_net_close(&socks[i]);
+    isc_mem_put(mctx, socks, nsocks * sizeof(*socks));
     isc_mem_put(mctx, queries, max_outstanding * sizeof(query_info));
     isc_mem_put(mctx, buckets, n_buckets * sizeof(ramp_bucket));
+    isc_mem_destroy(&mctx);
+    close(dummypipe[0]);
+    close(dummypipe[1]);
 }
 
 /* Find the ramp_bucket for queries sent at time "when" */
@@ -458,6 +491,35 @@ do_one_line(isc_buffer_t* lines, isc_buffer_t* msg)
     qid  = (q - queries) / nsocks;
     sock = (q - queries) % nsocks;
 
+    if (socks[sock].sending) {
+        if (perf_net_sockready(&socks[sock], dummypipe[0], TIMEOUT_CHECK_TIME) == -1) {
+            if (errno == EINPROGRESS) {
+                if (verbose) {
+                    perf_log_warning("network congested, packet sending in progress");
+                }
+            } else {
+                if (verbose) {
+                    char __s[256];
+                    perf_log_warning("failed to send packet: %s", perf_strerror_r(errno, __s, sizeof(__s)));
+                }
+            }
+            return (ISC_R_FAILURE);
+        }
+
+        ISC_LIST_UNLINK(instanding_list, q, link);
+        ISC_LIST_PREPEND(outstanding_list, q, link);
+        q->list = &outstanding_list;
+
+        num_queries_sent++;
+        num_queries_outstanding++;
+
+        q = ISC_LIST_HEAD(instanding_list);
+        if (!q)
+            return (ISC_R_NOMORE);
+        qid  = (q - queries) / nsocks;
+        sock = (q - queries) % nsocks;
+    }
+
     isc_buffer_clear(msg);
     result = perf_dns_buildrequest(NULL, (isc_textregion_t*)&used,
         qid, edns, dnssec, tsigkey, NULL, msg);
@@ -466,12 +528,34 @@ do_one_line(isc_buffer_t* lines, isc_buffer_t* msg)
 
     q->sent_timestamp = time_now;
 
+    switch (perf_net_sockready(&socks[sock], dummypipe[0], TIMEOUT_CHECK_TIME)) {
+    case 0:
+        if (verbose) {
+            perf_log_warning("failed to send packet: socket %d not ready", sock);
+        }
+        return (ISC_R_FAILURE);
+    case -1:
+        perf_log_warning("failed to send packet: socket %d readiness check timed out", sock);
+        return (ISC_R_FAILURE);
+    default:
+        break;
+    }
+
     base   = isc_buffer_base(msg);
     length = isc_buffer_usedlength(msg);
-    if (sendto(socks[sock], base, length, 0,
+    if (perf_net_sendto(&socks[sock], base, length, 0,
             &server_addr.type.sa, server_addr.length)
         < 1) {
-        perf_log_warning("failed to send packet: %s", strerror(errno));
+        if (errno == EINPROGRESS) {
+            if (verbose) {
+                perf_log_warning("network congested, packet sending in progress");
+            }
+        } else {
+            if (verbose) {
+                char __s[256];
+                perf_log_warning("failed to send packet: %s", perf_strerror_r(errno, __s, sizeof(__s)));
+            }
+        }
         return (ISC_R_FAILURE);
     }
 
@@ -521,15 +605,17 @@ try_process_response(unsigned int sockindex)
     int           n;
 
     packet_header = (uint16_t*)packet_buffer;
-    n             = recvfrom(socks[sockindex], packet_buffer, sizeof(packet_buffer),
-        0, NULL, NULL);
+    n             = perf_net_recv(&socks[sockindex], packet_buffer, sizeof(packet_buffer), 0);
     if (n < 0) {
         if (errno == EAGAIN || errno == EINTR) {
             return;
         } else {
-            perf_log_fatal("failed to receive packet: %s",
-                strerror(errno));
+            char __s[256];
+            perf_log_fatal("failed to receive packet: %s", perf_strerror_r(errno, __s, sizeof(__s)));
         }
+    } else if (!n) {
+        // Treat connection closed like try again until reconnection features are in
+        return;
     } else if (n < 4) {
         perf_log_warning("received short response");
         return;
@@ -588,6 +674,21 @@ num_scheduled(uint64_t time_since_start)
     }
 }
 
+static void
+handle_sigpipe(int sig)
+{
+    (void)sig;
+    switch (mode) {
+    case sock_tcp:
+    case sock_tls:
+        // if connection is closed it will generate a signal
+        perf_log_fatal("SIGPIPE received, connection(s) likely closed, can't continue");
+        break;
+    default:
+        break;
+    }
+}
+
 int main(int argc, char** argv)
 {
     int           i;
@@ -602,8 +703,19 @@ int main(int argc, char** argv)
     printf("DNS Resolution Performance Testing Tool\n"
            "Version " PACKAGE_VERSION "\n\n");
 
+    (void)SSL_library_init();
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+    SSL_load_error_strings();
+    OPENSSL_config(0);
+#endif
+
     setup(argc, argv);
 
+    if (pipe(dummypipe) < 0)
+        perf_log_fatal("creating pipe");
+
+    perf_os_handlesignal(SIGPIPE, handle_sigpipe);
+
     isc_buffer_init(&lines, input_data, sizeof(input_data));
 
     max_packet_size = edns ? MAX_EDNS_PACKET : MAX_UDP_PACKET;
@@ -676,8 +788,8 @@ end_loop:
 
     plotf = fopen(plotfile, "w");
     if (!plotf) {
-        perf_log_fatal("could not open %s: %s", plotfile,
-            strerror(errno));
+        char __s[256];
+        perf_log_fatal("could not open %s: %s", plotfile, perf_strerror_r(errno, __s, sizeof(__s)));
     }
 
     /* Print column headers */
@@ -709,6 +821,9 @@ end_loop:
     fclose(plotf);
     print_statistics();
     cleanup();
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+    ERR_free_strings();
+#endif
 
     return 0;
 }
